-- set name and namespace
addonname, PAR = ...

-- shortcut
local CPB = _G.C_PetBattles

-- lazy, easy condition evaluation functions
local function Any(v, t) for i = 1, #t do if (v == t[i]) then return true end end return false end
local function NotAny(v, t) for i = 1, #t do if (v == t[i]) then return false end end return true end

-- Debugging (remove hyphens from the following line to activate)
--local DEBUGGING = true

-- string constants
local RejectedAbilityString = "This ability is not supported by the Pet Accuracy Recorder due to Blizzard bugs."
local MultiTurnAbilityString = "Multi-turn abilities are not currently supported by Pet Accuracy Recorder."
local ConditionalAbilityString = "Effect %i with %s (Exp. %.2f%%): %.2f%%"
local NoDataString = "No data recorded for this ability yet."
local EffectFormatString = "Effect %i (Exp. %.2f%%): %.2f%%"
local SampleSizeString = "Sample Size: %i"
local ConditionNames = {
	[52] = "Chilled",
	[56] = "Darkness",
	[77] = "Bleeding",
	[82] = "Blinded",
}

-- multiple effect handling
local lastabilityID
--local lastturnIndex - possible future improvement
local lasteffectIndex

-- create event handling frame and register event(s)
local PAREventFrame = CreateFrame("Frame", "PAREventFrame")
PAREventFrame:RegisterEvent("ADDON_LOADED")
PAREventFrame:RegisterEvent("CHAT_MSG_PET_BATTLE_COMBAT_LOG")

-- reset all data
local function ResetPARData()
	PARData = {}
	PARData.Interface = select(4, GetBuildInfo())
	PARData.PlayerData = {}
	PARData.PlayerData.Miss = 0
	PARData.PlayerData.Hit = 0
	PARData.PlayerData.Total = 0
	PARData.EnemyData = {}
	PARData.EnemyData.Miss = 0
	PARData.EnemyData.Hit = 0
	PARData.EnemyData.Total = 0
end

-- get accuracy values and account for all known factors (including buffs, debuffs, level differences, and conditional ability states!)
local function PAR_GetAccuracy(abilityID, petOwner, petIndex, enemyOwner, enemyIndex)
	
	-- localize lasteffectIndex for now
	local leI = lasteffectIndex
	
	-- lasteffectIndex is optional, so account for that
	local startIndex = (leI + 1) or 1
	
	-- calculate AccuracyModifier using Blizzard's StateValues (alternate approach: use AccuracyAuras array)
	local AccuracyModifier
	if (CPB.GetPetType(petOwner, petIndex) == 7) then -- Elementals aren't affected by weathers.
		AccuracyModifier = CPB.GetStateValue(petOwner, petIndex, 41) + CPB.GetStateValue(petOwner, 0, 41)
	else
		AccuracyModifier = CPB.GetStateValue(petOwner, petIndex, 41) + CPB.GetStateValue(petOwner, 0, 41) + CPB.GetStateValue(0, 0, 41)
	end
	
	-- adjust accuracy if pets are not the same level and RETURN if level not found
	local LevelModifier = 0
	local petLevel = CPB.GetLevel(petOwner, petIndex)
	local enemyLevel = CPB.GetLevel(enemyOwner, enemyIndex)
	if (not petLevel) or (not enemyLevel) then
		if DEBUGGING then
			print("Pet levels could not be determined.")
		end
		return
	end
	if (petLevel < enemyLevel) then
		if DEBUGGING then
			print("Active pets are not the same level; adjusting predicted accuracy.")
		end
		LevelModifier = (enemyLevel - petLevel) * 2
	end
	
	-- Obtain raw accuracy value for ability by looping through possible turns/effects
	-- this code was heavily modified in r6 to account for abilities with multiple effects
	-- however it cannot handle multi-turn effects yet
	local RawAccuracy
	for effectIndex = startIndex, 10 do
		RawAccuracy = CPB.GetAbilityEffectInfo(abilityID, 1, effectIndex, "accuracy")
		if RawAccuracy then 
			if leI then lasteffectIndex = effectIndex end
			break
		end
	end
	
	-- RETURN if we don't find any accuracy value
	if not RawAccuracy then
		if DEBUGGING then
			print("Error, found no accuracy result for ability with ID " .. abilityID .. " and last effect index " .. lasteffectIndex .. ".")
		end
		return
	end
	
	-- check for conditional abilities and then account for their conditions' effects on accuracy
	local conditional = false
	local ConditionalModifier = 0
	if (PAR.ConditionalAbilities[id]) and (PAR.ConditionalAbilities[id][1] == lasteffectIndex) then
		-- format for PAR.ConditionalAbilities is [abilityID]={effectIndex, ConditionalCheckType, StateIDofCondition, ConditionalModifier}
		local conditionID = PAR.ConditionalAbilities[id][3]
		
		-- check condition on various
		if (PAR.ConditionalAbilities[id][2] == "t") then -- state should be checked on "Target" (enemyOwner, enemyIndex, conditionID)
			local value = CPB.GetStateValue(enemyOwner, enemyIndex, conditionID)
			if (value) and (value ~= 0) then conditional = true end
		elseif (PAR.ConditionalAbilities[id][2] == "c") then -- state should be checked on "Caster" (petOwner, petIndex, conditionID)
			local value = CPB.GetStateValue(petOwner, petIndex, conditionID)
			if (value) and (value ~= 0) then conditional = true end
		elseif (PAR.ConditionalAbilities[id][2] == "b") then -- state should be checked on "Both" (both of the above conditions)
			local firstvalue = CPB.GetStateValue(petOwner, petIndex, conditionID)
			local secondvalue = CPB.GetStateValue(enemyOwner, enemyIndex, conditionID)
			if (firstvalue) and (secondvalue) and (firstvalue ~= 0) and (secondvalue ~= 0) then conditional = true end
		elseif (PAR.ConditionalAbilities[id][2] == "w") then -- state should be checked on "Weather" (0, 0, conditionID)
			local value = CPB.GetStateValue(0, 0, conditionID)
			if (value) and (value ~= 0) then conditional = true end
		elseif (PAR.ConditionalAbilities[id][2] == "ft") then -- state should be checked on "Field Target" (enemyOwner, 0, conditionID)
			local value = CPB.GetStateValue(enemyOwner, 0, conditionID)
			if (value) and (value ~= 0) then conditional = true end
		elseif (PAR.ConditionalAbilities[id][2] == "fc") then -- state should be checked on "Field Caster" (petOwner, 0, conditionID)
			local value = CPB.GetStateValue(petOwner, 0, conditionID)
			if (value) and (value ~= 0) then conditional = true end
		end
		
		-- if the condition is active, set up the accuracy modifier and increment the effect index to support the condition in writing form
		if conditional then ConditionalModifier = PAR.ConditionalAbilities[id][4] end
	end
	
	-- combine Accuracy and make sure it stays within values that make sense
	local totalAccuracy = RawAccuracy + AccuracyModifier - LevelModifier + ConditionalModifier
	if (totalAccuracy < 0) then totalAccuracy = 0 end
	if (totalAccuracy > 100) then totalAccuracy = 100 end
	
	return totalAccuracy, conditional
end

-- hook into tooltips and provide recorded accuracy data
local function SharedPetBattleAbilityTooltip_SetAbility_Hook(self, PBAI)
	if (not self.Description) or (not PBAI) then return end
	
	-- obtain ability description and ability ID
	local description = self.Description:GetText()
	local id = PBAI.abilityID or C_PetBattles.GetAbilityInfo(PBAI.petOwner, PBAI.petIndex, PBAI.abilityIndex)
	
	-- escape if ability ID not found
	if not id then return end
	
	-- string building variables
	local outputString = {}
	local finalString
	
	-- assemble string now if we aren't recording data for this ability
	if Any(id, PAR.RejectedAbilities) then finalString = RejectedAbilityString end
	if Any(id, PAR.MultiTurnAbilities) then finalString = MultiTurnAbilityString end
	
	-- if the data is being recorded then
	if not finalString then
		-- statistics variables
		accuracytable = {}
		expectedAccuracy = {}
		actualAccuracy = {}
		
		-- construct data table for current ability
		for x, y in next, PARData.PlayerData do
			if (type(y) == "table") and (PARData.PlayerData[x][id]) then
				for e, r in next, PARData.PlayerData[x][id] do
					if (type(r) == "table") then
						-- create table sections if needed
						if not accuracytable[e] then
							accuracytable[e] = {}
							expectedAccuracy[e] = 0
							actualAccuracy[e] = 0
						end
						
						-- record table/effect/accuracy data
						accuracytable[e][x] = {}
						accuracytable[e][x].Total = PARData.PlayerData[x][id][e].Total
						accuracytable[e][x].Hit = PARData.PlayerData[x][id][e].Hit
						accuracytable[e][x].Miss = PARData.PlayerData[x][id][e].Miss
						
						-- tally accuracy data into table/effect area
						if not accuracytable[e].Total then
							accuracytable[e].Total = accuracytable[e][x].Total
						else
							accuracytable[e].Total = accuracytable[e].Total + accuracytable[e][x].Total
						end
						if not accuracytable[e].Hit then
							accuracytable[e].Hit = accuracytable[e][x].Hit
						else
							accuracytable[e].Hit = accuracytable[e].Hit + accuracytable[e][x].Hit
						end
						if not accuracytable[e].Miss then
							accuracytable[e].Miss = accuracytable[e][x].Miss
						else
							accuracytable[e].Miss = accuracytable[e].Miss + accuracytable[e][x].Miss
						end
						
						-- tally accuracy data into table area
						if not accuracytable.Total then
							accuracytable.Total = accuracytable[e].Total
						else
							accuracytable.Total = accuracytable.Total + accuracytable[e].Total
						end
						if not accuracytable.Hit then
							accuracytable.Hit = accuracytable[e].Hit
						else
							accuracytable.Hit = accuracytable.Hit + accuracytable[e].Hit
						end
						if not accuracytable.Miss then
							accuracytable.Miss = accuracytable[e].Miss
						else
							accuracytable.Miss = accuracytable.Miss + accuracytable[e].Miss
						end
					end
				end
			end
		end
		
		-- RETURN if no data
		if (not accuracytable.Total) or (accuracytable.Total == 0) then
			finalString = NoDataString
		else
		
			-- traverse current data table and calculate statistics
			for e, r in next, accuracytable do
				if (type(r) == "table") then
					for x, y in next, accuracytable[e] do
						if (type(y) == "table") then
							local prevalence = accuracytable[e][x].Total / accuracytable[e].Total
							expectedAccuracy[e] = expectedAccuracy[e] + ((x / 100) * prevalence)
							actualAccuracy[e] = actualAccuracy[e] + ((accuracytable[e][x].Hit / accuracytable[e][x].Total) * prevalence)
						end
					end
				end
			end
			
			-- assemble outputString
			for i = 1, 20 do
				if accuracytable[i] then
					if (i > 10) then
						tinsert(outputString, format(ConditionalAbilityString, (i - 10), ConditionNames[PAR.ConditionalAbilities[id][3]], expectedAccuracy[i] * 100, actualAccuracy[i] * 100))
					else
						tinsert(outputString, format(EffectFormatString, i, expectedAccuracy[i] * 100, actualAccuracy[i] * 100))
					end
				end
			end
			
			-- add sample size line
			if accuracytable[1] then tinsert(outputString, format(SampleSizeString, accuracytable[1].Total)) end
			
			-- finalize string
			finalString = table.concat(outputString, "|n")
		end
	end
	
	-- provide statistics to user in tooltip
	self.Description:SetText(string.format("%s|n|n|cFF03FBFF%s|r", description, finalString))
end

-- event handling
local function PAREventFrame_OnEvent(self, event, message, ...)
	if (event == "ADDON_LOADED") and (message == addonname) then
		
		-- create initial SavedVariables if not found
		if (not PARData) then
			ResetPARData()
		end
		
		-- prompt user to reset data if outdated
		if (not PARData.Interface) or (PARData.Interface ~= select(4, GetBuildInfo())) then
			StaticPopupDialogs["RESET_PAR"] = {
				text = "World of Warcraft has been patched. This will make your Pet Accuracy Recorder data relatively meaningless. Reset data now?",
				button1 = "Yes",
				button2 = "No",
				OnAccept = function()
					ResetPARData()
					print("PAR: Resetting all recorded data.")
				end,
				timeout = 0,
				whileDead = true,
				hideOnEscape = true,
			}
			StaticPopup_Show("RESET_PAR")
		end
		
		-- unregister event now that addon has loaded
		PAREventFrame:UnregisterEvent("ADDON_LOADED")
	elseif (event == "CHAT_MSG_PET_BATTLE_COMBAT_LOG") then
	
		-- RETURN if not a miss, buff/debuff application, or damaging attack
		if (not strmatch(message, " applied ")) and (not strmatch(message, " missed ")) and (not strmatch(message, " dealt ")) then
			if DEBUGGING then
				print("Not a miss, application, or damage.")
			end
			return
		end
		
		-- retrieve abilityID from message (first abilityID is only relevant one, so ignoring possiblity of others)
		local startpos = strfind(message, "battlePetAbil")
		local _, endpos = strfind(message, "|h|r")
		local _, abilityID, _ = strsplit(":", strsub(message, startpos, endpos))
		abilityID = tonumber(abilityID)
		
		-- RETURN if abilityID matches any RejectedAbilities, matches any MultiTurnAbilities, or is missing entirely
		if (not abilityID) or (PAR.RejectedAbilities[abilityID]) or (PAR.MultiTurnAbilities[abilityID]) then
			if DEBUGGING and abilityID then
				print("Ability with ID " .. abilityID .. " will not be recorded.")
			elseif DEBUGGING then
				print("Ability ID not found!")
			end
			return
		end
		
		-- Find out which player is doing the attack
		-- " enemy " means you did the attack
		-- " your " means the enemy did the attack
		local petOwner, enemyOwner
		if strmatch(message, " enemy ") then
			petOwner = 1
			enemyOwner = 2
		elseif strmatch(message, " your ") then
			petOwner = 2
			enemyOwner = 1
		else
			if DEBUGGING then
				print("Couldn't figure out petOwner/enemyOwner.")
			end
			return
		end
		
		-- find active pet and RETURN if missing
		local petIndex = CPB.GetActivePet(petOwner)
		local enemyIndex = CPB.GetActivePet(enemyOwner)
		if not (petIndex and enemyIndex) then
			if DEBUGGING then
				print("Active pet could not be found.")
			end
			return
		end
		
		-- RETURN if opposite pet has an aura from MissAuras
		local numAuras = CPB.GetNumAuras(enemyOwner, enemyIndex)
		if (numAuras) and (numAuras > 0) then
			for auraIndex = 1, numAuras do
				local currentAura = CPB.GetAuraInfo(enemyOwner, enemyIndex, auraIndex)
				if Any(currentAura, PAR.MissAuras) then
					if DEBUGGING then
						print("There is an Aura on the enemy pet that causes misses.")
					end
					return
				end
			end	
		end
		
		-- determine if ability was the same as the last ability used and reset or increment effect index if so
		if (not lastabilityID) or (lastabilityID ~= abilityID) then
			lastabilityID = abilityID
			lasteffectIndex = 0
		elseif (lastabilityID == abilityID) then
			lasteffectIndex = lasteffectIndex + 1
		end
		
		-- RETURN if last effect index is invalid
		if (not lasteffectIndex) or (lasteffectIndex > 9) then
			if DEBUGGING then
				print("Last effect index was out of bounds.")
			end
			return
		end
		
		-- RETURN if Arcane Storm is present and the current ability/effect combination can cause a Stun or Root
		local ArcaneStorm = CPB.GetStateValue(enemyOwner, enemyIndex, 54)
		if (ArcaneStorm) and (ArcaneStorm ~= 0) then 
			if ((PAR.StunAbilityEffects[abilityID]) and (PAR.StunAbilityEffects[abilityID] == lasteffectIndex)) or ((PAR.RootAbilityEffects[abilityID]) and (PAR.RootAbilityEffects[abilityID] == lasteffectIndex)) then
				if DEBUGGING then
					print("Skipping! The presence of Arcane Storm causes ability ID " .. abilityID .. " with effect ID " .. lasteffectIndex .. " to always miss.")
				end
				return
			end
		end
		
		-- obtain total accuracy value and check if conditional
		local totalAccuracy, conditional = PAR_GetAccuracy(abilityID, petOwner, petIndex, enemyOwner, enemyIndex, lasteffectIndex)
		
		-- RETURN if total accuracy value not found
		if not totalAccuracy then
			if DEBUGGING then
				print("Could not obtain total accuracy for ability with ID " .. abilityID .. " and last effect index " .. lasteffectIndex .. ".")
			end
			return
		end
		
		-- adjust lasteffectIndex for ease of recording conditionals (restoring it after this massive IF block)
		if conditional then lasteffectIndex = lasteffectIndex + 10 end
		
		-- record data, separate into miss and hit (dealt/applied)
		-- table structure is PARData[totalAccuracy][abilityID].Miss (and Hit and Total)
		-- then follow up by looking at PARData[totalAccuracy].Miss
		-- and lastly PARData.Miss
		if (petOwner == 1) then
			
			-- create table structure if empty previously
			if (not PARData.PlayerData[totalAccuracy]) then
				PARData.PlayerData[totalAccuracy] = {}
				PARData.PlayerData[totalAccuracy].Miss = 0
				PARData.PlayerData[totalAccuracy].Hit = 0
				PARData.PlayerData[totalAccuracy].Total = 0
			end
			if (not PARData.PlayerData[totalAccuracy][abilityID]) then
				PARData.PlayerData[totalAccuracy][abilityID] = {}
				PARData.PlayerData[totalAccuracy][abilityID].Miss = 0
				PARData.PlayerData[totalAccuracy][abilityID].Hit = 0
				PARData.PlayerData[totalAccuracy][abilityID].Total = 0
			end
			if (not PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex]) then
				PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex] = {}
				PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex].Miss = 0
				PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex].Hit = 0
				PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex].Total = 0
			end
			
			if (strmatch(message, " applied ")) or (strmatch(message, " dealt ")) then
				
				-- add a "Hit" result
				PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex].Hit = PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex].Hit + 1
				PARData.PlayerData[totalAccuracy][abilityID].Hit = PARData.PlayerData[totalAccuracy][abilityID].Hit + 1
				PARData.PlayerData[totalAccuracy].Hit = PARData.PlayerData[totalAccuracy].Hit + 1
				PARData.PlayerData.Hit = PARData.PlayerData.Hit + 1
				
				-- increment total number of recorded attacks
				PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex].Total = PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex].Total + 1
				PARData.PlayerData[totalAccuracy][abilityID].Total = PARData.PlayerData[totalAccuracy][abilityID].Total + 1
				PARData.PlayerData[totalAccuracy].Total = PARData.PlayerData[totalAccuracy].Total + 1
				PARData.PlayerData.Total = PARData.PlayerData.Total + 1
			elseif (strmatch(message, " missed ")) then
				
				-- add a "Miss" result
				PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex].Miss = PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex].Miss + 1
				PARData.PlayerData[totalAccuracy][abilityID].Miss = PARData.PlayerData[totalAccuracy][abilityID].Miss + 1
				PARData.PlayerData[totalAccuracy].Miss = PARData.PlayerData[totalAccuracy].Miss + 1
				PARData.PlayerData.Miss = PARData.PlayerData.Miss + 1
				
				-- increment total number of recorded attacks
				PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex].Total = PARData.PlayerData[totalAccuracy][abilityID][lasteffectIndex].Total + 1
				PARData.PlayerData[totalAccuracy][abilityID].Total = PARData.PlayerData[totalAccuracy][abilityID].Total + 1
				PARData.PlayerData[totalAccuracy].Total = PARData.PlayerData[totalAccuracy].Total + 1
				PARData.PlayerData.Total = PARData.PlayerData.Total + 1
			else
				if DEBUGGING then
					print("Bizarre error with string matching on first pet owner.")
				end
				return
			end
		elseif (petOwner == 2) then
		
			-- create table structure if empty previously
			if (not PARData.EnemyData[totalAccuracy]) then
				PARData.EnemyData[totalAccuracy] = {}
				PARData.EnemyData[totalAccuracy].Miss = 0
				PARData.EnemyData[totalAccuracy].Hit = 0
				PARData.EnemyData[totalAccuracy].Total = 0
			end
			if (not PARData.EnemyData[totalAccuracy][abilityID]) then
				PARData.EnemyData[totalAccuracy][abilityID] = {}
				PARData.EnemyData[totalAccuracy][abilityID].Miss = 0
				PARData.EnemyData[totalAccuracy][abilityID].Hit = 0
				PARData.EnemyData[totalAccuracy][abilityID].Total = 0
			end
			if (not PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex]) then
				PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex] = {}
				PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex].Miss = 0
				PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex].Hit = 0
				PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex].Total = 0
			end
			
			if (strmatch(message, " applied ")) or (strmatch(message, " dealt ")) then
				
				-- add a "Hit" result
				PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex].Hit = PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex].Hit + 1
				PARData.EnemyData[totalAccuracy][abilityID].Hit = PARData.EnemyData[totalAccuracy][abilityID].Hit + 1
				PARData.EnemyData[totalAccuracy].Hit = PARData.EnemyData[totalAccuracy].Hit + 1
				PARData.EnemyData.Hit = PARData.EnemyData.Hit + 1
				
				-- increment total number of recorded attacks
				PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex].Total = PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex].Total + 1
				PARData.EnemyData[totalAccuracy][abilityID].Total = PARData.EnemyData[totalAccuracy][abilityID].Total + 1
				PARData.EnemyData[totalAccuracy].Total = PARData.EnemyData[totalAccuracy].Total + 1
				PARData.EnemyData.Total = PARData.EnemyData.Total + 1
			elseif (strmatch(message, " missed ")) then
				
				-- add a "Miss" result
				PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex].Miss = PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex].Miss + 1
				PARData.EnemyData[totalAccuracy][abilityID].Miss = PARData.EnemyData[totalAccuracy][abilityID].Miss + 1
				PARData.EnemyData[totalAccuracy].Miss = PARData.EnemyData[totalAccuracy].Miss + 1
				PARData.EnemyData.Miss = PARData.EnemyData.Miss + 1
				
				-- increment total number of recorded attacks
				PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex].Total = PARData.EnemyData[totalAccuracy][abilityID][lasteffectIndex].Total + 1
				PARData.EnemyData[totalAccuracy][abilityID].Total = PARData.EnemyData[totalAccuracy][abilityID].Total + 1
				PARData.EnemyData[totalAccuracy].Total = PARData.EnemyData[totalAccuracy].Total + 1
				PARData.EnemyData.Total = PARData.EnemyData.Total + 1
			else
				if DEBUGGING then
					print("Bizarre error with string matching on second pet owner.")
				end
				return
			end
		else
			if DEBUGGING then
				print("Bizarre error with pet ownership.")
			end
			return
		end
		
		-- if conditional, revert lasteffectIndex back to normal
		if conditional then lasteffectIndex = lasteffectIndex - 10 end
	end
end

-- set our event handler function
PAREventFrame:SetScript("OnEvent", PAREventFrame_OnEvent)

-- hook ability tooltip functions
hooksecurefunc("SharedPetBattleAbilityTooltip_SetAbility", SharedPetBattleAbilityTooltip_SetAbility_Hook)

-- slash command handling
SLASH_PETACCURACYRECORDER1 = "/PAR"
SLASH_PETACCURACYRECORDER2 = "/petaccuracy"
SlashCmdList["PETACCURACYRECORDER"] = function(message)
	
	-- lowercase message for evaluation
	local msg = string.lower(message)
	
	-- RETURN if no data or invalid data or user requested it
	if (msg == "reset") or (not PARData.PlayerData.Total) or (not PARData.EnemyData.Total) or (PARData.PlayerData.Total == 0) or (PARData.EnemyData.Total == 0) then
		if (msg == "reset") then
			print("PAR: Resetting all recorded data.")
		else
			print("PAR: Data is invalid or missing. Resetting all recorded data. Next time, recording more data before running this command.")
		end
		ResetPARData()
		return
	end
	
	--
	-- DO STATS FOR PLAYER DATA
	--
	
	-- first find ExpectedOverallAccuracy
	local ExpectedOverallAccuracy = 0
	
	-- use totalAccuracy * prevalence, where prevalence is calculated as (PARData[totalAccuracy].Total / PARData.Total)
	for k, v in next, PARData.PlayerData do
		if (k) and (type(v) == "table") then
			ExpectedOverallAccuracy = ExpectedOverallAccuracy + ((k / 100) * (PARData.PlayerData[k].Total / PARData.PlayerData.Total))
		end
	end
	
	-- next find TrueOverallAccuracy, calculated as (PARData.Hit / PARData.Total)
	local TrueOverallAccuracy = PARData.PlayerData.Hit / PARData.PlayerData.Total
	
	-- save results to data table
	PARData.PlayerData.Expected = ExpectedOverallAccuracy
	PARData.PlayerData.True = TrueOverallAccuracy
	
	-- show results to user
	print(format("Expected Accuracy for Player: %.2f%%", PARData.PlayerData.Expected * 100))
	print(format("Actual Accuracy for Player: %.2f%%", PARData.PlayerData.True * 100))
	print(format("Sample Size: %i attacks", PARData.PlayerData.Total))
	
	-- do statistical analysis (obtain std deviation and perform t-test) in future version
	--[[for k, v in next, PARData do
		if (k) and (type(v) == "table") then
			ExpectedOverallAccuracy = ExpectedOverallAccuracy + ((k / 100) * (PARData[k].Total / PARData.Total))
		end
	end]]--
	
	--
	-- DO STATS FOR ENEMY DATA
	--
	
	-- first find ExpectedOverallAccuracy
	local ExpectedOverallAccuracy = 0
	
	-- use totalAccuracy * prevalence, where prevalence is calculated as (PARData[totalAccuracy].Total / PARData.Total)
	for k, v in next, PARData.EnemyData do
		if (k) and (type(v) == "table") then
			ExpectedOverallAccuracy = ExpectedOverallAccuracy + ((k / 100) * (PARData.EnemyData[k].Total / PARData.EnemyData.Total))
		end
	end
	
	-- next find TrueOverallAccuracy, calculated as (PARData.Hit / PARData.Total)
	local TrueOverallAccuracy = PARData.EnemyData.Hit / PARData.EnemyData.Total
	
	-- save results to data table
	PARData.EnemyData.Expected = ExpectedOverallAccuracy
	PARData.EnemyData.True = TrueOverallAccuracy
	
	-- show results to user
	print(format("Expected Accuracy for Enemy: %.2f%%", PARData.EnemyData.Expected * 100))
	print(format("Actual Accuracy for Enemy: %.2f%%", PARData.EnemyData.True * 100))
	print(format("Sample Size: %i attacks", PARData.EnemyData.Total))
	
	-- do statistical analysis (obtain std deviation and perform t-test) in future version
	--[[for k, v in next, PARData do
		if (k) and (type(v) == "table") then
			ExpectedOverallAccuracy = ExpectedOverallAccuracy + ((k / 100) * (PARData[k].Total / PARData.Total))
		end
	end]]--
end